////////////////////////////////////////////////////////////////////////////////
//
// Paella
// Copyright (C) 2015 - Thomas FORGIONE, Emilie JALRAS, Marion LENFANT, Thierry MALON, Amandine PAILLOUX
// Authors :
//     Thomas FORGIONE
//     Emilie JALRAS
//     Marion LENFANT
//     Thierry MALON
//     Amandine PAILLOUX
//
// This file is part of the project Paella
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
////////////////////////////////////////////////////////////////////////////////
#ifndef ANIMATION_HPP
#define ANIMATION_HPP

//////////////////////////////////////////////
/// \defgroup animation Animation module
///
/// Contains all the classes and functions to animate a mesh
///
/// The main binary of this module is AnimatedMain. Its execution is in two steps
///  - the subdivision of the skeleton
///    - a window with the skeleton is opened. You might use your left click
///      to add rotation points on the branch of the skeleton (for example, you may
///      add elbows of knees to your skeleton)
///    - once you're done, you should use your right click to select a root to the trees
///      You should right click near an extremity of a skeleton (and not a junction)
///
///  - then, a new window is opened in fullscreen mode, with the OpenGL
///    rendering of your skeletons. The colors of the mesh are chosen at
///    random, and mapped to each part of the skeleton.
///    On this new window, you will be able to control the camera, and the value of
///    the angles for the rotations.
///
///    If a joystick is detected, it will be automatically used as a controller
///    for the camera. Else, you should use your keyboard (Z,Q,S,D for moving
///    and the mouse for changing camera angles).
///
///    You can use the key-arrows of your keyboard to move in the menu
///    displayed on the upper-left part of the window, and + and - keys to change the value
///    of the angles.
///
/// Note that this program is meant to be used with skeletons where the z component of each
/// is 0.
//////////////////////////////////////////////

#include <Animation/AnimatedMesh.hpp>
#include <Click.hpp>
#include <Menu.hpp>

#endif // ANIMATION_HPP
