////////////////////////////////////////////////////////////////////////////////
//
// Paella
// Copyright (C) 2015 - Thomas FORGIONE, Emilie JALRAS, Marion LENFANT, Thierry MALON, Amandine PAILLOUX
// Authors :
//     Thomas FORGIONE
//     Emilie JALRAS
//     Marion LENFANT
//     Thierry MALON
//     Amandine PAILLOUX
//
// This file is part of the project Paella
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
////////////////////////////////////////////////////////////////////////////////
#ifndef CALIBRATION_HPP
#define CALIBRATION_HPP

/////////////////////////////////////////////////////
/// \defgroup calibration Calibration
///
/// This module provides a program Calibration that allows to get the intrinsic
/// parameters of the camera.
///
/// You may use this program with some parameters :
///     - -w <board_width>         the number of inner corners per one of board dimension
///     - -h <board_height>        the number of inner corners per another board dimension
///     - [-pt <pattern>]          the type of pattern: chessboard or circles grid
///     - [-n <number_of_frames>]  the number of frames to use for calibration
///                                (if not specified, it will be set to the number
///                                 of board views actually available)
///     - [-d <delay>]             a minimum delay in ms between subsequent attempts to capture a next view
///                                (used only for video capturing)
///     - [-s <squareSize>]        square size in some user-defined units (1 by default)
///     - [-o <out_camera_params>] the output filename for intrinsic [and extrinsic] parameters
///     - [-op]                    write detected feature points
///     - [-oe]                    write extrinsic parameters
///     - [-zt]                    assume zero tangential distortion
///     - [-a <aspectRatio>]       fix aspect ratio (fx/fy)
///     - [-p]                     fix the principal point at the center
///     - [-v]                     flip the captured images around the horizontal axis
///     - [-V]                     use a video file, and not an image list, uses
///                                [input_data] string for the video file name
///     - [-su]                    show undistorted images after calibration
///     - [input_data]             input data, one of the following:
///                                 - text file with a list of the images of the board
///                                   the text file can be generated with imagelist_creator
///                                 - name of video file with a video of the board
///                                if input_data not specified, a live view from the camera is used
///
/// For example, if you have a chessboard with 9 rows and 7 columns, you may use
/// \code
/// bin/Calibration -w 8 -h 6 <path_to_video> -o <path_to_output_xml_file>
/// \endcode
/////////////////////////////////////////////////////

#endif // CALIBRATION_HPP
