////////////////////////////////////////////////////////////////////////////////
//
// Paella
// Copyright (C) 2015 - Thomas FORGIONE, Emilie JALRAS, Marion LENFANT, Thierry MALON, Amandine PAILLOUX
// Authors :
//     Thomas FORGIONE
//     Emilie JALRAS
//     Marion LENFANT
//     Thierry MALON
//     Amandine PAILLOUX
//     Simone GASPARINI
//
// This file is part of the project Paella
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
////////////////////////////////////////////////////////////////////////////////
#ifndef _CHESSBOARDCAMERATRACKER_HPP_
#define _CHESSBOARDCAMERATRACKER_HPP_

#include "Camera.hpp"
#include "utility.hpp"

/**
 * \ingroup calibration
 * \brief Class that wraps a chessboard detector and compute the pose mat
 */
class ChessboardCameraTracker
{
public:
    /**
     * \brief Default constructor
     */
    ChessboardCameraTracker() = default;


    /**
    * It detects a chessboard inside an image and if found it returns the pose of the camera wrt the chessboard
    *
    * @param[in,out] img the original image
    * @param[out] pose the pose of the camera
    * @param[in] cam the camera
    * @param[in] boardSize the size of the chessboard to detect
    * @return true if the chessboard has been found
    */
    bool process( cv::Mat &img, cv::Mat &pose, const Camera & cam, const cv::Size &boardSize);


};

#endif /*_CHESSBOARDCAMERATRACKER_HPP_*/
