////////////////////////////////////////////////////////////////////////////////
//
// Paella
// Copyright (C) 2015 - Thomas FORGIONE, Emilie JALRAS, Marion LENFANT, Thierry MALON, Amandine PAILLOUX
// Authors :
//     Thomas FORGIONE
//     Emilie JALRAS
//     Marion LENFANT
//     Thierry MALON
//     Amandine PAILLOUX
//
// This file is part of the project Paella
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
////////////////////////////////////////////////////////////////////////////////
#ifndef BASE_HPP
#define BASE_HPP

#include <vector>
#include <Geometry/Vector.hpp>

namespace geo
{

// Forward declaration of class Spline
template<typename... Types>
class Spline;

//////////////////////////////////////////////////////////////////
/// \brief Base function for splines
///
/// \param j Number of the base function for the spline
///
/// \param n Degree of the spline
///
/// \param t Time when the base function in computed. Must be between 0 and 1
///
/// \param nodes Vector of nodes
///
/// \return the value of the base function in t
///
//////////////////////////////////////////////////////////////////
float base(int j, int n, float t, std::vector<float> const& nodes);

//////////////////////////////////////////////////////////////////
/// \relates Spline
/// \brief Compute the length of a spline
///
/// \param spline Spline to compute the length.
///
/// \return the length of the spline
//////////////////////////////////////////////////////////////////
float splineLenght(Spline<Vector3<float>, float> const& spline);

} // end namespace geo

#include "Spline.hpp"

#endif // BASE_HPP
