////////////////////////////////////////////////////////////////////////////////
//
// Paella
// Copyright (C) 2015 - Thomas FORGIONE, Emilie JALRAS, Marion LENFANT, Thierry MALON, Amandine PAILLOUX
// Authors :
//     Thomas FORGIONE
//     Emilie JALRAS
//     Marion LENFANT
//     Thierry MALON
//     Amandine PAILLOUX
//
// This file is part of the project Paella
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
////////////////////////////////////////////////////////////////////////////////
#include <iostream>

#include "TupleFunctions.hpp"

namespace pae
{

namespace detail
{

template<std::size_t I, std::size_t J, typename... Types>
struct loadFromStreamHelper
{
    void operator()(std::tuple<Types...>& tuple, std::stringstream& stream)
    {
        stream >> std::get<J>(tuple);
        loadFromStreamHelper<I-1,J+1,Types...>{}(tuple, stream);
    }
};

template<std::size_t J, typename... Types>
struct loadFromStreamHelper<0,J,Types...>
{
    void operator()(std::tuple<Types...>& tuple, std::stringstream& stream)
    {
        // Nothing to do here
    }
};

template<typename... Types>
void loadFromStream(std::tuple<Types...>& tuple, std::stringstream& stream)
{
    loadFromStreamHelper<std::tuple_size<std::tuple<Types...>>::value, 0, Types...>{}(tuple, stream);
}

template<std::size_t I, std::size_t J, typename... Types>
struct printHelper
{
    std::ostream& operator()(std::ostream& out, std::tuple<Types...> const& tuple)
    {
        out << std::get<J>(tuple) << " ";
        return printHelper<I-1,J+1,Types...>{}(out, tuple);
    }
};

template<std::size_t J, typename... Types>
struct printHelper<0,J,Types...>
{
    std::ostream& operator()(std::ostream& out, std::tuple<Types...> const& tuple)
    {
        // Nothing to do here
        return out;
    }
};

template<typename... Types>
std::ostream& operator<<(std::ostream& out, std::tuple<Types...> const& tuple)
{
    return printHelper<std::tuple_size<std::tuple<Types...>>::value,0,Types...>{}(out, tuple);
}

} // namespace detail

} // namespace pae
