////////////////////////////////////////////////////////////////////////////////
//
// Paella
// Copyright (C) 2015 - Thomas FORGIONE, Emilie JALRAS, Marion LENFANT, Thierry MALON, Amandine PAILLOUX
// Authors :
//     Thomas FORGIONE
//     Emilie JALRAS
//     Marion LENFANT
//     Thierry MALON
//     Amandine PAILLOUX
//
// This file is part of the project Paella
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
////////////////////////////////////////////////////////////////////////////////
#include <cmath>
#include <iostream>

#include <Geometry/Base.hpp>

namespace geo
{

float base(int j, int n, float t, std::vector<float> const& nodes)
{
    if (n == 0)
    {
        return static_cast<float>(nodes[j] <= t && (t < nodes[j+1] || nodes[j+1] == 1));
    }
    else
    {
        float sum = 0;
        float denom = nodes[j+n] - nodes[j];

        if (std::abs(denom) > 0.001)
            sum += (t - nodes[j])/denom * base(j, n-1, t, nodes);

        denom = nodes[j+n+1] - nodes[j+1];

        if (std::abs(denom) > 0.001)
            sum += (nodes[j+n+1] - t)/denom * base(j+1, n-1, t, nodes);

        return sum;
    }
}

float splineLenght(Spline<Vector3<float>, float> const& spline)
{
    float len = 0;
    for (unsigned int i = 0; i < spline.controlPoints.size()-1; i++)
    {
        auto point1 = std::get<0>(spline.controlPoints[i]);
        auto point2 = std::get<0>(spline.controlPoints[i+1]);
        len += (point1-point2).norm();
    }
    return len;
}

} // end namespace geo
