////////////////////////////////////////////////////////////////////////////////
//
// Paella
// Copyright (C) 2015 - Thomas FORGIONE, Emilie JALRAS, Marion LENFANT, Thierry MALON, Amandine PAILLOUX
// Authors :
//     Thomas FORGIONE
//     Emilie JALRAS
//     Marion LENFANT
//     Thierry MALON
//     Amandine PAILLOUX
//
// This file is part of the project Paella
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
////////////////////////////////////////////////////////////////////////////////
#include <Geometry/Rotation.hpp>

namespace geo
{

Rotation::Rotation() : theta(0), phi(0), psi(0)
{

}

void Rotation::operator()() const
{
    glTranslatef(center.x(), center.y(), center.z());
    glRotatef(theta, 1, 0, 0);
    glRotatef(phi,   0, 1, 0);
    glRotatef(psi,   0, 0, 1);
    glTranslatef(-center.x(), -center.y(), -center.z());
}

float const& Rotation::operator[](std::size_t i) const
{
    switch (i)
    {
        case 0: return theta;
        case 1: return phi;
        case 2: return psi;
        default: throw std::out_of_range{"Only 3 items on rotation"};
    }
}

float& Rotation::operator[](std::size_t i)
{
    return const_cast<float&>(static_cast<const Rotation*>(this)->operator[](i));
}

} // namespace geo
