////////////////////////////////////////////////////////////////////////////////
//
// Paella
// Copyright (C) 2015 - Thomas FORGIONE, Emilie JALRAS, Marion LENFANT, Thierry MALON, Amandine PAILLOUX
// Authors :
//     Thomas FORGIONE
//     Emilie JALRAS
//     Marion LENFANT
//     Thierry MALON
//     Amandine PAILLOUX
//
// This file is part of the project Paella
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <string>

#include "Geometry/Vector.hpp"
#include "Geometry/Spline.hpp"
#include "Geometry/Mesh.hpp"
#include "Geometry/Point.hpp"

#include "Meshing/Skeleton3D.hpp"

int main(int argc, char *argv[])
{
    using namespace pae;
    using namespace pae::detail;

    geo::Circle<float> c;
    c.points.push_back(std::make_pair(7,geo::Vector3<float>{0.0f,2.0f,-1.0f}));
    c.points.push_back(std::make_pair(8,geo::Vector3<float>{0.0f,1.0f,-1.0f}));
    c.points.push_back(std::make_pair(9,geo::Vector3<float>{0.0f,0.0f,-1.0f}));

    c.points.push_back(std::make_pair(0,geo::Vector3<float>{0.0f,0.0f, 1.0f}));
    c.points.push_back(std::make_pair(1,geo::Vector3<float>{0.0f,1.0f, 1.0f}));
    c.points.push_back(std::make_pair(2,geo::Vector3<float>{0.0f,2.0f, 1.0f}));
    c.points.push_back(std::make_pair(3,geo::Vector3<float>{0.0f,3.0f, 1.0f}));
    c.points.push_back(std::make_pair(4,geo::Vector3<float>{0.0f,4.0f, 1.0f}));
    c.points.push_back(std::make_pair(5,geo::Vector3<float>{0.0f,4.0f,-1.0f}));
    c.points.push_back(std::make_pair(6,geo::Vector3<float>{0.0f,3.0f,-1.0f}));

    auto v = sortedPoints(std::array<double,4>{0,0,1,0},
                          std::vector<geo::Circle<float>>{c},
                          std::vector<std::pair<geo::Point<float>, geo::Point<float>>>{});

    for (auto& element : v)
    {
        for (auto& e : element)
        {
            std::cout << e.first << " " << e.second << std::endl;
        }
        std::cout << std::endl;
    }

    return 0;
}
